#!/bin/sh

SERVER_IP_ADDRESS=172.16.2.101
PROTCOL='https'
USER_NAME=''
PASSWORD=''

PRODUCT_PATH='products/x11a'
URL_PATH="${PROTCOL}://${SERVER_IP_ADDRESS}/${PRODUCT_PATH}"

BOOTLOADER_IMG_PREFIX='u-boot-a600-v'
BOOTLOADER_IMG_SUFFIX='.imx'
KERNEL_IMG_PREFIX='uImage-a600-v'
KERNEL_IMG_SUFFIX=''
DTB_IMG_PREFIX='armadilo-640-v'
DTB_IMG_SUFFIX='.dtb'
APP_ARCHIVE_PREFIX='app-v'
APP_ARCHIVE_SUFFIX='.tar.xz'

VERSIONSDIR='/var/x11a/versions'
LOCKDIR='/var/lock/auto_image_update'
TMPDIR=''
FILE_IS_UPDATED=false

WGET_OPS=''
if [ "$USER_NAME" ]; then
    WGET_OPS="--http-user=$USER_NAME"
    if [ "$PASSWORD" ]; then
        WGET_OPS="$WGET_OPS --http-password=$PASSWORD"
    fi
fi

log()
{
    logger -p user.$1 -t "$0[$$]" -- "$2"
}

ledctrl()
{
    local LEDCLASS="/sys/class/leds"
    local led="$1"
    shift
    local op="$1"
    shift

    case "${op}" in
        "blink_on")
            local interval="$1"

            echo "timer" > "${LEDCLASS}/${led}/trigger"
            echo $interval > "${LEDCLASS}/${led}/delay_on"
            echo $interval > "${LEDCLASS}/${led}/delay_off"
            ;;
        "blink_off")
            echo "none" > "${LEDCLASS}/${led}/trigger"
            echo 0 > "${LEDCLASS}/${led}/brightness"
            ;;
    esac
}

die()
{
    log err "$1"

    if [ "$TMPDIR" ]; then
        cd
        umount $TMPDIR
        rm -rf $TMPDIR
    fi

    ledctrl red blink_off
    rmdir $LOCKDIR

    exit 1
}

bootloader_update_x1_iotg3()
{
    x1-bootloader-install "$1"
}

kernel_update_x1_iotg3()
{
    mount -t vfat /dev/mmcblk2p1 /mnt && \
    cp "$1" /mnt/uImage && \
    umount /mnt
}

dtb_update_x1_iotg3()
{
    mount -t vfat /dev/mmcblk2p1 /mnt && \
    cp "$1" "$DTB_PATH" && \
    umount /mnt
}

bootloader_update_a600series()
{
    dd if="$1" of=/dev/mmcblk0 \
       bs=1k seek=1 conv=fsync
}

kernel_update_a600series()
{
    mv "$1" /boot/uImage
}

dtb_update_a600series()
{
    mv "$1" /boot/a640.dtb
}

python_update()
{
    # pip3 install pip-review
    pip-review --auto
}

get_latest_filename()
{
    local prefix="$1"
    local suffix="$2"
    local url="$3"
    # for example, if
    #   prefix="uImage-a600-v"
    #   suffix=""
    #   url="https://download.atmark-techno.com/armadillo-640/image/"
    # then get_latest_filename puts
    #   "uImage-a600-v4.14-at21"
    # to stdout as of 2020-04-21.

    wget $WGET_OPS -q -O - "$url" | html2 | grep @href | \
        if [ -n "${prefix}" ]; then
            grep "${prefix}"
        else
            sed '' # pass through
        fi | \
        if [ -n "${suffix}" ]; then
            grep "${suffix}"
        else
            sed '' # pass through
        fi | grep -v ".md5" | \
        sort -V | tail -n 1 | sed -r 's/^.*@href=(.*)$/\1/'
}

check_and_set_version()
{
    local version="$1"

    mkdir -p "${VERSIONSDIR}"

    if find "${VERSIONSDIR}" -mindepth 1 | \
       grep -q "^${VERSIONSDIR}/${version}$"; then
        if tail -n 1 "${VERSIONSDIR}/${version}" | \
           grep -q "^install OK"; then
            return 0 # already installed
        else
            return 1 # previous installation failed
        fi
    fi

    touch "${VERSIONSDIR}/${version}"
    return 2 # then installation start
}

complete_set_version()
{
    echo "install OK" >> "${VERSIONSDIR}/$1"
}

incomplete_set_version()
{
    local version=$1
    shift
    local msg="$@"

    echo "install NG${msg}" >> "${VERSIONSDIR}/$1"
}

download_and_md5sum()
{
    local url_path="$1"
    local filename="$2"

    wget $WGET_OPS -q "${url_path}/${filename}" -O "${filename}" || \
        log err "download ${filename} failed"

    wget $WGET_OPS -q "${url_path}/${filename}.md5" -O "${filename}.md5"
    if [ $? -ne 0 ]; then
        md5sum "${filename}" > "${filename}.md5"
        rm -f "${filename}"

        wget $WGET_OPS -q "${url_path}/${filename}" -O "${filename}"
    fi

    md5sum -c "${filename}.md5"
}

update()
{
    local fail
    local msg
    local update

    python_update

    local LATEST_BOOTLOADER_NAME="$(get_latest_filename "${BOOTLOADER_IMG_PREFIX}" "${BOOTLOADER_IMG_SUFFIX}" "${URL_PATH}")"
    local LATEST_KERNEL_NAME="$(get_latest_filename "${KERNEL_IMG_PREFIX}" "${KERNEL_IMG_SUFFIX}" "${URL_PATH}")"
    local LATEST_DTB_NAME="$(get_latest_filename "${DTB_IMG_PREFIX}" "${DTB_IMG_SUFFIX}" "${URL_PATH}")"
    local LATEST_APP_ARCHIVE_NAME="$(get_latest_filename "${APP_ARCHIVE_PREFIX}" "${APP_ARCHIVE_SUFFIX}" "${URL_PATH}")"

    msg=""
    fail=1
    if [ -n "${LATEST_BOOTLOADER_NAME}" ]; then
        if check_and_set_version "${LATEST_BOOTLOADER_NAME}"; then
            : "nothing to do"
        else
            log info "update bootloader to ${LATEST_BOOTLOADER_NAME}"

            download_and_md5sum "${URL_PATH}" \
                "${LATEST_BOOTLOADER_NAME}" || \
                download_and_md5sum "${URL_PATH}" \
                    "${LATEST_BOOTLOADER_NAME}" # retry
            if [ $? -ne 0 ]; then
                log err "download ${URL_PATH}/${LATEST_BOOTLOADER_NAME} failed"
                msg="${msg}: donwload failed"
            else
                $bootloader_update "${LATEST_BOOTLOADER_NAME}"
                if [ $? -eq 0 ]; then
                    fail=0
                else
                    log err "update ${LATEST_BOOTLOADER_NAME} failed"
                    msg="${msg}: copy failed"
                fi
            fi

            if [ $fail -eq 0 ]; then
                complete_set_version "${LATEST_BOOTLOADER_NAME}"
                FILE_IS_UPDATED=true
            else
                incomplete_set_version "${LATEST_BOOTLOADER_NAME}" "$msg"
            fi
        fi
    fi

    msg=""
    fail=1
    if [ -n "${LATEST_KERNEL_NAME}" ]; then
        if check_and_set_version "${LATEST_KERNEL_NAME}"; then
            : "nothing to do"
        else
            log info "update kernel to ${LATEST_KERNEL_NAME}"

            download_and_md5sum "${URL_PATH}" \
                "${LATEST_KERNEL_NAME}" || \
                download_and_md5sum "${URL_PATH}" \
                    "${LATEST_KERNEL_NAME}" # retry
            if [ $? -ne 0 ]; then
                log err "download ${URL_PATH}/${LATEST_KERNEL_NAME} failed"
                msg="${msg}: donwload failed"
            else
                $kernel_update "${LATEST_KERNEL_NAME}"
                if [ $? -eq 0 ]; then
                    fail=0
                else
                    log err "update ${LATEST_KERNEL_NAME} failed"
                    msg="${msg}: copy failed"
                fi
            fi

            if [ $fail -eq 0 ]; then
                complete_set_version "${LATEST_KERNEL_NAME}"
                FILE_IS_UPDATED=true
            else
                incomplete_set_version "${LATEST_KERNEL_NAME}" "$msg"
            fi
        fi
    fi

    msg=""
    fail=1
    if [ -n "${LATEST_DTB_NAME}" ]; then
        if check_and_set_version "${LATEST_DTB_NAME}"; then
            : "nothing to do"
        else
            log info "update dtb to ${LATEST_DTB_NAME}"

            download_and_md5sum "${URL_PATH}" \
                "${LATEST_DTB_NAME}" || \
                download_and_md5sum "${URL_PATH}" \
                    "${LATEST_DTB_NAME}" # retry
            if [ $? -ne 0 ]; then
                log err "download ${URL_PATH}/${LATEST_DTB_NAME} failed"
                msg="${msg}: donwload failed"
            else
                $dtb_update "${LATEST_DTB_NAME}"
                if [ $? -eq 0 ]; then
                    fail=0
                else
                    log err "update ${LATEST_DTB_NAME} failed"
                    msg="${msg}: copy failed"
                fi
            fi

            if [ $fail -eq 0 ]; then
                complete_set_version "${LATEST_DTB_NAME}"
                FILE_IS_UPDATED=true
            else
                incomplete_set_version "${LATEST_DTB_NAME}" "$msg"
            fi
        fi
    fi

    msg=""
    fail=1
    if [ -n "${LATEST_APP_ARCHIVE_NAME}" ]; then
        if check_and_set_version "${LATEST_APP_ARCHIVE_NAME}"; then
            : "nothing to do"
        else
            log info "update applications by extracting ${LATEST_APP_ARCHIVE_NAME}"

            download_and_md5sum "${URL_PATH}" \
                "${LATEST_APP_ARCHIVE_NAME}" || \
                download_and_md5sum "${URL_PATH}" \
                    "${LATEST_APP_ARCHIVE_NAME}" # retry
            if [ $? -ne 0 ]; then
                log err "download ${URL_PATH}/${LATEST_APP_ARCHIVE_NAME} failed"
                msg="${msg}: donwload failed"
            else
                tar xf "${LATEST_APP_ARCHIVE_NAME}" -C /
                if [ $? -eq 0 ]; then
                    fail=0
                else
                    log err "update ${LATEST_APP_ARCHIVE_NAME} failed"
                    msg="${msg}: extract archive failed"
                fi
            fi

            if [ $fail -eq 0 ]; then
                complete_set_version "${LATEST_APP_ARCHIVE_NAME}"
                FILE_IS_UPDATED=true
            else
                incomplete_set_version "${LATEST_APP_ARCHIVE_NAME}" "$msg"
            fi
        fi
    fi
}

model="$(cat /proc/device-tree/model)" > /dev/null
case "$model" in
    *"Atmark Techno Armadillo-6"*)
        bootloader_update=bootloader_update_a600series
        kernel_update=kernel_update_a600series
        dtb_update=dtb_update_a600series
        case "$model" in
            *"Armadillo-64"*)
                DTB_PATH="/boot/a640.dtb"
                ;;
            *"Armadillo-61"*)
                DTB_PATH="/boot/a610.dtb"
                ;;
            *)
                die "unknown model($model)"
                ;;
        esac
        ;;
    *"Atmark-Techno Armadillo-X1"*"miniaml"*)
        die "detect minimal model($model)"
        ;;
    *"Atmark-Techno Armadillo-X1"*|*"Atmark-Techno Armadillo-IoT"*"G3"*)
        bootloader_update=bootloader_update_x1_iotg3
        kernel_update=kernel_update_x1_iotg3
        dtb_update=dtb_update_x1_iotg3
        case "$model" in
            *"Armadillo-X1 Board"*)
                DTB_PATH="/mnt/armadillo_x1.dtb"
                ;;
            *"Armadillo-X1L Board"*)
                DTB_PATH="/mnt/armadillo_x1l.dtb"
                ;;
            *"Armadillo-IoT"*"G3 Board"*)
                DTB_PATH="/mnt/armadillo_iotg_g3.dtb"
                ;;
            *"Armadillo-IoT"*"G3 M1 Board"*)
                DTB_PATH="/mnt/armadillo_iotg_g3_m1.dtb"
                ;;
            *"Armadillo-IoT"*"G3 W2 Board"*)
                DTB_PATH="/mnt/armadillo_iotg_g3_w2.dtb"
                ;;
            *)
                die "unknown model($model)"
                ;;
        esac
        ;;
    *)
        die "unknown model($model)"
        ;;
esac

mkdir -p $LOCKDIR || die "making LOCKDIR($LOCKDIR) is failed"

ledctrl red blink_on 100

TMPDIR=$(mktemp -d /tmp/web_software_update.XXXXXXXX)
if [ -z "$TMPDIR" ]; then
    die "mktemp failed"
fi

cd $TMPDIR

update

cd
rm -rf $TMPDIR


ledctrl red blink_off

rmdir $LOCKDIR

if [ "$FILE_IS_UPDATED" = true ]; then
    log info "reboot"
    reboot
fi
