import sys
import os
import csv
import traceback
import argparse
from peewee import *

# CSVデータ1行の要素数
COLUMN_NUM = 11

# データベースファイルのオープン
db = SqliteDatabase("address.db")

# 住所クラスの定義
class Address(Model):
    zipcode = CharField()
    pref = CharField()
    city = CharField()
    street = CharField()

    class Meta:
        database = db

# Addressテーブルの作成
db.create_tables([Address])

def add(args):

    """ データベースへのデータの追加

    Args:
        args: コマンド引数
            args.file: CSVファイル名

    """

    with open(args.file) as csvfile:
        csvreader = csv.reader(csvfile)
        count = 0
        try:
            # トランザクションの開始
            with db.transaction():
                # CSVファイルから1行読み込む
                # 10行目まで読み込む
                for row in csvreader:
                    # 要素数が不足している場合、次行にスキップ
                    if len(row) < COLUMN_NUM:
                        continue
                    if count >= 10:
                        break

                    # データベースへのデータの追加(INSERT)
                    Address.create(zipcode=row[1], pref=row[4], city=row[3], street=row[2])

                    count += 1

                # トランザクション終了 データをコミット
                db.commit()
        except:
            # エラー発生の場合データベースをロールバック
            db.rollback()
            print(traceback.format_exc())

def show(args):

    """ データベースに保存されているデータの表示

    Args:
        args: コマンド引数

    """

    for address in Address.select():
        print("%s %s %s %s %s" \
                % (address.id, address.zipcode, address.pref, address.city, address.street))

def update(args):

    """ データベースに保存されているデータを変更する

    Args:
        args: コマンド引数
            args.id: 変更したいデータのid
            args.column: 変更したいデータのカラム名
            args.val: 変更後の値

    """

    try:
        # トランザクションの開始
        with db.transaction():
            # idを指定してデータを取得
            address = Address.get(id=args.id)

            # 指定されたカラム名毎に値を変更
            if args.column == "zipcode":
                address.zipcode = args.val
            elif args.column == "pref":
                address.pref = args.val
            elif args.column == "city":
                address.city = args.val
            elif args.column == "street":
                address.street = args.val

            # 変更を保存してトランザクション終了
            address.save()
            db.commit()
    except:
        # エラー発生の場合データベースをロールバック
        db.rollback()
        print(traceback.format_exc())

def delete(args):

    """ データベースに保存されているデータを削除

    Args:
        args: コマンド引数
            args.id: 削除したいデータのid

    """

    try:
        # トランザクションの開始
        with db.transaction():
            # idを指定してデータを取得
            address = Address.get(id=args.id)

            # データを削除
            address.delete_instance()

            # トランザクション終了
            db.commit()
    except:
        # エラー発生の場合データベースをロールバック
        db.rollback()
        print(traceback.format_exc())

def commandline_parser():

    """ コマンドオプションとハンドラを設定する

    """

    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers()

    # addコマンド
    add_command = subparsers.add_parser("add")
    add_command.add_argument("-F", "--file", required=True, help="csv file")
    add_command.set_defaults(handler=add)

    # showコマンド
    show_command = subparsers.add_parser("show")
    show_command.set_defaults(handler=show)

    # updateコマンド
    update_command = subparsers.add_parser("update")
    update_command.add_argument("-I", "--id", required=True, help="id number")
    update_command.add_argument("-C", "--column", required=True, help="column name")
    update_command.add_argument("-V", "--val", required=True, help="new value")
    update_command.set_defaults(handler=update)

    # deleteコマンド
    delete_command = subparsers.add_parser("delete")
    delete_command.add_argument("-I", "--id", required=True, help="id number")
    delete_command.set_defaults(handler=delete)

    return parser

if __name__ == "__main__":
    parser = commandline_parser()
    args = parser.parse_args()
    if hasattr(args, 'handler'):
        # ハンドラが登録されていれば実行
        args.handler(args)
    else:
        # 未知のサブコマンドの場合はヘルプを表示
        parser.print_help()
